/***************************************************************************
                          k8ktest.h  -  description
                             -------------------
    begin                : Sun Jul 16 2000
    copyright            : (C) 2000 by Joris Struyve
    email                : j.struyve@village.uunet.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* CVS information :
 *
 * $Id: k8ktest.h,v 1.12 2000/08/01 18:12:49 js Exp $
 *
 */


/* QT includes */
#ifndef K8KTEST
#define K8KTEST
#define NO_DEBUG
#include <qwindowsstyle.h>
#include <qtextstream.h>
#include <qstring.h>
#include <qmainwindow.h>
#include <qstatusbar.h>
#include <qmenubar.h>
#include <qtoolbutton.h>
#include <qtoolbar.h>
#include <qpixmap.h>
#include <qmultilinedit.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qdialog.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qtooltip.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <qfile.h>
#include <qscrollbar.h>


/* other includes */
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <errno.h>
#include <stdarg.h>
#include <signal.h>
#include <sys/io.h>
#include <k8000++.h>

/* defines */
#define VERSION "K8ktest : 1.0.0 (Development Release 3)."

/* prototypes */
void Signal(int i);
void Debug(char *format,...);

/* Class defintions */
class ApplicationWinow;
class AboutDialog;
class StatusBar;
class MenuBar;
class ToolBar;
class DebugDialog;
class ScrollText;
class AnalogOutput;
class DigitalIo;
class AnalogInput;
class OptionsDialog;
class Settings;

/*************************************************************************************
* ApplicationWindow definition
*
*
*
**************************************************************************************/

class ApplicationWindow: public QMainWindow
{
Q_OBJECT
public:
	ApplicationWindow(QString con);
	~ApplicationWindow();
	
	void InsertDebugText(QString debugmessage);
	
	Settings *settings;

public slots:
	void ShowAbout();
	void RestoreSettings();
	void SaveSettings();
	void NullSlot();
	
	void SetUnsetDebug();
	void ShowDebugDialog();
	void HideDebugDialog();
	void ShowHideDebugDialog();
	
	void ShowOptionDialog();
	
	void Scan();
	void AllOutput();
	void AllIntput();
	void SetMinimum();
	void SetMaximum();

	
signals:

private:
	void SetNewOptions();
	
	DebugDialog *debugdialog;
	OptionsDialog *optionsdialog;
	
	ToolBar *toolbar;
	StatusBar *statusbar;
	MenuBar *menu;
	
	QTimer *timer;
	
	AnalogOutput *analogoutput[9];
	AnalogInput  *analoginput[4];
	DigitalIo    *digitalio[16];
	
	QFrame *mainframe;
	QFrame *analogoframe;
	QFrame *analogiframe;
	QFrame *digitalioframe;
	QFrame *buttonframe;
	
	QPushButton *quit;
	QPushButton *set_min;
	QPushButton *set_max;
	QPushButton *alli;
	QPushButton *allo;
	
	bool debugstate;
	bool debugwindowstate;
	
};


/*************************************************************************************
* AboutDialog definition
*
*
*
**************************************************************************************/

class AboutDialog : public QDialog
{
Q_OBJECT
public:
	AboutDialog( QWidget *parent, const char *name );
	~AboutDialog();
	
	void resizeEvent(QResizeEvent * r_event);
private:
	QPushButton *ok;
};


/*************************************************************************************
* DebugDialog definition
*
*
*
**************************************************************************************/

class DebugDialog: public QDialog
{
Q_OBJECT
public:
	DebugDialog(QWidget *parent,char *label);
	~DebugDialog();
	
	void InsertDebugText(QString debugmessage);
	void resizeEvent(QResizeEvent * r_event);
	
public slots:
	void hideEvent(QHideEvent *h);
	void ClearMessages();

signals:
	void Closed();

private:
	QPushButton *clear;
	QPushButton *hidebutton;
	ScrollText *debugmessages;
	QWidget *par;

};

/*************************************************************************************
* OptionsDialog definition
*
*
*
**************************************************************************************/

class OptionsDialog: public QDialog
{
Q_OBJECT
public:
	OptionsDialog(QWidget *parent,char *label);
	~OptionsDialog();
	
	void resizeEvent(QResizeEvent * r_event);
	int GetRefreshrate(){return refreshrate->text().toInt();};
	int GetI2cBusDelay(){return i2cbusdelay->text().toInt();};
	int GetLptport(){return lptport->text().toInt();};
	
public slots:
	void hideEvent(QHideEvent *h);

signals:
	void Closed();

private:
	QWidget *par;
	
	QPushButton *ok;
	QPushButton *cancel;
	
	QLineEdit *i2cbusdelay;
	QLineEdit *refreshrate;
	QLineEdit *lptport;
	
	QLabel *l_i2cbusdelay;
	QLabel *l_refreshrate;
	QLabel *l_lptport;
	
};

/*************************************************************************************
* ScrollText definition
*
*
*
**************************************************************************************/

class ScrollText: public QMultiLineEdit
{
Q_OBJECT
public:
	ScrollText(QWidget *parent,char *label);
	~ScrollText();

	void append(QString str);
	
private:
	QWidget *par;

};

/*************************************************************************************
* StatusBar definition
*
*
*
**************************************************************************************/

class StatusBar: public QStatusBar
{
Q_OBJECT
public:
	StatusBar(QWidget *parent,char *name);
	~StatusBar();

public slots:
	void SetStatusString(QString connectstring);
	void Blink();
	void BlinkTimer();

private:
	QWidget *par;

	QLabel *statusstring;
	QLabel *blinker;
	QTimer *timer;

};

/*************************************************************************************
* MenuBar definition
*
*
*
**************************************************************************************/

class MenuBar: public QMenuBar
{
Q_OBJECT
public:
	MenuBar(QWidget *parent,char *name);
	~MenuBar();

	bool GetDebugState()
		{return this->view->isItemChecked(dbggenerate);};
	bool GetDebugWindowState()
		{return this->view->isItemChecked(dbgwindow);};

public slots:
	void SetDebugState(bool state)
		{this->view->setItemChecked(dbggenerate,state);};
	void SetDebugWindowState(bool state)
		{this->view->setItemChecked(dbgwindow,state);};

private:
	QWidget *par;
	
	QPopupMenu *file;
	QPopupMenu *view;
	QPopupMenu *options;
	QPopupMenu *help;
	
	int dbgwindow;
	int dbggenerate;
};

/*************************************************************************************
* ToolBar definition
*
*
*
**************************************************************************************/

class ToolBar: public QToolBar
{
Q_OBJECT
public:
	ToolBar(ApplicationWindow *parent,char *name);
	~ToolBar();

private:
	QWidget *par;
	
	QToolButton *quit;	
	QToolButton *refresh;
	QToolButton *options;
};

/*************************************************************************************
* DigitalIo definition
*
*
*
**************************************************************************************/
class DigitalIo: public QFrame
{
Q_OBJECT
public:
	DigitalIo(bool value,int io,QString label,QWidget *parent,char *name);
	~DigitalIo();

	void SetGeometry(int x,int y,int w,int h,int s);
	bool GetValue();
	bool GetIoType();

public slots:
	void SetValue(bool value);
	void ChangeValue();
	void Refresh();
	void SetIoType(bool type);

signals:
	void ValueChanged(bool value);

private:
	QWidget *par;
	
	QFrame *container;
	QToolButton *val;
	QLabel *cardlabel;
	QButtonGroup *io_config;
	QRadioButton *i_type;
	QRadioButton *o_type;

	int io_channel;
};

/*************************************************************************************
* AnalogOutput definition
*
*
*
**************************************************************************************/
class AnalogOutput: public QFrame
{
Q_OBJECT
public:
	AnalogOutput(int min,int max, int linestep,int pagestep,int value,int io,QString label,
				QWidget *parent,char *name);
	~AnalogOutput();

	void SetGeometry(int x,int y,int w,int h,int s);
	int GetValue();

public slots:
	void SetValue(int value);
	void ChangeValue(int value);
	void Refresh();

signals:
	void ValueChanged(int value);

private:
	QWidget *par;
	
	QFrame *container;
	QScrollBar *val;
	QLabel *valuelabel;
	QLabel *cardlabel;
	
	int io_channel;

};

/*************************************************************************************
* AnalogInput definition
*
*
*
**************************************************************************************/
class AnalogInput: public QFrame
{
Q_OBJECT
public:
	AnalogInput(int min,int max, int linestep,int pagestep,int value,int io,QString label,
				QWidget *parent,char *name);
	~AnalogInput();
	void SetGeometry(int x,int y,int w,int h,int s);
	int GetValue();

public slots:
	void ChangeValue(int value);
	void Refresh();

signals:
	void ValueChanged(int value);

private:
	QWidget *par;
	
	QFrame *container;
	QScrollBar *val;
	QLabel *valuelabel;
	QLabel *cardlabel;
	
	int io_channel;

};

/*************************************************************************************
* Settings definition
*
*
*
**************************************************************************************/
class Settings
{

public:
	Settings();
	~Settings();
	
	void SaveSettings();
	void RestoreSettings();

	int i2cbusdelay;
	int refreshrate;
	int lptport;
	
	bool iotype[16];
	bool iostate[16];
	int aostate[9];
	
private:
	QFile *configfile;
};

#endif
